package org.netkernel.lang.javascript;

import java.util.List;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.ContextFactory;
import org.mozilla.javascript.ImporterTopLevel;
import org.mozilla.javascript.RhinoException;
import org.mozilla.javascript.Script;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.mozilla.javascript.xmlimpl.XML;
import org.netkernel.container.IKernel;
import org.netkernel.layer0.meta.IEndpointArgumentMeta;
import org.netkernel.layer0.meta.impl.EndpointArgumentMetaImpl;
import org.netkernel.layer0.nkf.INKFParameters;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.impl.NKFContextImpl;
import org.netkernel.layer0.util.RequestScopeClassLoader;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;
import org.netkernel.urii.impl.NetKernelException;
import org.w3c.dom.Node;

public class JavascriptPrototype extends StandardAccessorImpl
{
		private int mVerbs;
		
		public JavascriptPrototype()
		{	declareThreadSafe();
			declareTolerantParameters();
		}
		
		public String toString()
		{	return mMeta.getName();
		}
		
		public void onCommissionEndpoint(IKernel aKernel, ISpace aContext) throws Exception
		{	INKFParameters params = this.getParameters();
		
			Object script = getParameter("script");
			if (script instanceof IIdentifier)
			{	declareName(((IIdentifier)script).toString());
			}
			else
			{	declareName("Javascript");
			}
		
			String verbsString = (String)params.getParamValue("verbs");
			mVerbs = Utils.parseVerbList(verbsString);
			if (mVerbs!=0)
			{	declareSupportedVerbs(mVerbs);
			}
			
			//setup from representations
			List<Object> paramValues=params.getParamValues("fromRepresentation");
			Class[] fromRepresentations = new Class[paramValues.size()];
			for (int i=0; i<fromRepresentations.length; i++)
			{	String fromRepString=(String)paramValues.get(i);
				Class fromRep=aContext.getClassLoader().loadClass(fromRepString);
				fromRepresentations[i]=fromRep;
			}
			IEndpointArgumentMeta arg= new EndpointArgumentMetaImpl(IEndpointArgumentMeta.PRIMARY_PART,null,null,null,null,false,fromRepresentations);
			declareArgument(arg);
			
			//setup to representations
			 paramValues=params.getParamValues("toRepresentation");
			for (int i=0; i<paramValues.size(); i++)
			{	String toRepString=(String)paramValues.get(i);
				Class toRep=aContext.getClassLoader().loadClass(toRepString);
				declareSourceRepresentation(toRep);
			}

			super.onCommissionEndpoint(aKernel, aContext);
		}
		
		@Override
		public void onRequest(INKFRequestContext context) throws Exception
		{	if (context.getThisRequest().getVerb()==INKFRequestReadOnly.VERB_META)
			{	super.onMeta(context);
			}
			else
			{	String sourceIdentifier=context.getThisRequest().getArgumentValue(this.getMeta().getName());
				((NKFContextImpl)context).setRuntimeSourceIdentifier(sourceIdentifier);
				try
				{	Context ctx=ContextFactory.getGlobal().enterContext();
					ClassLoader CL=new RequestScopeClassLoader(context.getKernelContext().getThisKernelRequest().getRequestScope());
					ctx.setApplicationClassLoader(CL);
					Scriptable scope =new ImporterTopLevel(ctx);
					Object wrappedContext = Context.javaToJS(context, scope);
					ScriptableObject.putProperty(scope, "context", wrappedContext);
					Object wrappedLoader = Context.javaToJS(new LibraryLoader(context, ctx, scope), scope);
					ScriptableObject.putProperty(scope, "contextloader", wrappedLoader);
					Script s=context.source("param:script", Script.class);
					s.exec(ctx, scope);
					
					Object resp=((NKFContextImpl)context).getResponse().getRepresentation();
					if (resp instanceof XML)
					{
						XML rx=(XML)resp;
						Node n=rx.toDomNode();
						JavascriptRuntime.recurseCleanNode(n);
						rx.setParentScope(null);
					}					
				}
				catch(RhinoException e)
				{	NetKernelException nke=new NetKernelException("JavascriptException", "line: "+e.lineNumber()+"  "+e.getMessage(), e);
					throw nke;
				}
				finally
				{	Context.exit();
				}
			}
		}
		
}